﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;

namespace TestWPF
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
    }

    /// <summary>
    /// Univerzalni trida pro data
    /// </summary>
    public class TableData : ICloneable
    {

        /// <summary>
        /// Pretizeni funkce Clone
        /// </summary>
        /// <returns></returns>
        object ICloneable.Clone()
        {

            return this.Clone();

        }

        /// <summary>
        /// Vlastni funkce Clone
        /// </summary>
        /// <returns></returns>
        public TableData Clone()
        {

            TableData T = (TableData)this.MemberwiseClone();  //--klonovani tridy

            T.ColumnData = new Dictionary<string, Dictionary<string, object>>();  //--na dictionary vytvorime novou instanci

            //--a rucne zkopirujeme vsechny polozky z Dictionary (Dictionary se musi vzdy kopirovat rucne, navic s vlastnosti object je to slozitejsi, aby se neprenasela reference)
            foreach (KeyValuePair<string, Dictionary<string, object>> Entry in this.ColumnData) T.ColumnData.Add(Entry.Key, this.CloneDictionaryCloningValues<string, object>(Entry.Value));

            return T;

        }

        /// <summary>
        /// Klonovani Dictionary s objektem
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="original"></param>
        /// <returns></returns>
        public Dictionary<TKey, TValue> CloneDictionaryCloningValues<TKey, TValue>(Dictionary<TKey, TValue> Original)
        {

            Dictionary<TKey, TValue> T = new Dictionary<TKey, TValue>();

            foreach (KeyValuePair<TKey, TValue> Entry in Original) T.Add(Entry.Key, (TValue)Entry.Value);

            return T;

        }

        public DateTime Date { get; set; }

        public string Sloupec1 { get; set; }

        //--zasobnik se vsemi hodnotami potrebny ke zpetne vazbe pri zmene dat
        public Dictionary<string, Dictionary<string, object>> ColumnData = new Dictionary<string, Dictionary<string, object>>();

        /// <summary>
        /// Nacteni hodnoty ze zasobniku podle daneho Id a Type
        /// </summary>
        /// <param name="Id"></param>
        /// <param name="Type"></param>
        /// <returns>Vrati nulovou hodnotu pokud se dany Typ klic nenajde nebo je ulozena v zasobniku</returns>
        public object GetData(string Id, string Type)
        {

            if (this.ColumnData.ContainsKey(Id) && this.ColumnData[Id].ContainsKey(Type)) return this.ColumnData[Id][Type];

            return null;

        }

        /// <summary>
        /// Nacteni double hodnoty pro dane Id
        /// </summary>
        /// <param name="Id"></param>
        /// <param name="Type"></param>
        /// <returns>Pokud se nepodari parsovat hodnotu nebo hodnota neni, vraci null</returns>
        public virtual double? GetDoubleVal(string Id)
        {

            if (this.ColumnData.ContainsKey(Id) && this.ColumnData[Id].ContainsKey("Val")) return (double?)this.ColumnData[Id]["Val"];

            return null;

        }

        /// <summary>
        /// Nacteni textoveho retezce pro dane Id
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public string GetStringVal(string Id)
        {

            if (this.ColumnData.ContainsKey(Id) && this.ColumnData[Id].ContainsKey("Val")) return this.ColumnData[Id]["Val"].ToString();

            return null;

        }

        /// <summary>
        /// Nastaveni double hodnoty pro dane Id
        /// </summary>
        /// <param name="Id"></param>
        /// <param name="Val"></param>
        public void SetDoubleVal(string Id, double? Val)
        {

            if (!this.ColumnData.ContainsKey(Id)) ColumnData[Id] = new Dictionary<string, object>();

            ColumnData[Id]["Val"] = Val;

        }

        /// <summary>
        /// Nastaveni string hodnoty pro dane Id
        /// </summary>
        /// <param name="Id"></param>
        /// <param name="Val"></param>
        public void SetStringVal(string Id, string Val)
        {

            if (!this.ColumnData.ContainsKey(Id)) ColumnData[Id] = new Dictionary<string, object>();

            ColumnData[Id]["Val"] = Val;

        }

        /// <summary>
        /// Nastaveni dat pro dane Id
        /// </summary>
        /// <param name="Id"></param>
        /// <param name="Data"></param>
        public void SetData(string Id, Dictionary<string, object> Data)
        {

            if (!this.ColumnData.ContainsKey(Id)) this.ColumnData[Id] = new Dictionary<string, object>();

            this.ColumnData[Id] = Data;

        }

        /// <summary>
        /// Overeni, zdali jsou pro dane datum nejaka data
        /// </summary>
        /// <returns></returns>
        public bool HaveData()
        {

            foreach (KeyValuePair<string, Dictionary<string, object>> DataIn in this.ColumnData)
            {

                if (DataIn.Value.Count > 0) return true;

            }

            return false;

        }

    }

}
